<!DOCTYPE html>
<html>
<head>
    <title>Profile - {{ username }}</title>
</head>
<body>
    <h1>User Profile</h1>
    <p>Username: {{ username }}</p>
    <p>Joined: {{ profile_data.joined }}</p>
    <p>Posts: {{ profile_data.posts }}</p>
    
    <!-- Context from middleware -->
    <div>
        <h2>Site Info</h2>
        <p>Site Name: {{ site_name }}</p>
        <p>App Version: {{ app_version }}</p>
        <p>Current User: {{ user.name }}</p>
    </div>
</body>
</html> 