<!DOCTYPE html>
<html>
<head>
    <title>{{ title }}</title>
</head>
<body>
    <h1>{{ title }}</h1>
    <p>{{ message }}</p>
    
    <!-- Context from middleware -->
    <div>
        <h2>Site Info</h2>
        <p>Site Name: {{ site_name }}</p>
        <p>App Version: {{ app_version }}</p>
        <p>User: {{ user.name }} (ID: {{ user.id }})</p>
    </div>
    
    <!-- Request object from middleware -->
    <div>
        <h2>Request Info</h2>
        <p>Method: {{ request.method }}</p>
        <p>Path: {{ request.path }}</p>
    </div>
</body>
</html> 