from nexios.types import Scope
from enum import Enum

class MatchStatus(Enum):
    NONE = 0
    PARTIAL = 1
    FULL = 2
def get_route_path(scope: Scope) -> str:
    
    path: str = scope["path"]
    root_path = scope.get("root_path", "")
    if not root_path:
        return path

    if not path.startswith(root_path):
        return path

    if path == root_path:
        return ""


    return path
